#!/usr/local/bin/perl -w

#BEGIN {
#    if ($^O ne "MSWin32")
#    {
#        require IPC::Shareable;
#        IPC::Shareable->import();  # assuming you would not be passing arguments to "use Module"
#    }
#}

$0 = "Tracker_agent";

###########################################
################ Read Args ################
###########################################
use Getopt::Long;

my $argnodisplay = 0;
my $argnoprogressbar = 0;
my $conffile = "";
my ($input_file, $nodisplay, $noprogressbar, $help);
$nodisplay=0;
$noprogressbar=0;
GetOptions (
   "input=s" => \$input_file,
   "nodisplay+" => \$nodisplay,
   "noprogressbar+" => \$noprogressbar,
   "help" =>\$help
);

if (!defined ($input_file)) {
   $conffile = "tracker_agent.conf";
} else {
   $conffile = $input_file ;
}

if ($help) {
   print "  --help or -h          Display this help\n";
   print "  --nodisplay           Display nothing when execute\n";
   print "  --noprogressbar       Hide progress bar\n";
   print "  -i                    The name of input config file\n";
   exit;
}

if ($nodisplay>0) {
   $argnodisplay = 1;
}
if ($noprogressbar>0) {
   $argnoprogressbar = 1;
}

if ($^O eq "MSWin32") {
   $argnoprogressbar = 1;
}

###########################################
############### Load modules ##############
###########################################

if ($argnodisplay == 0) {
   use diagnostics;
}
use strict;
use Config;
use Net::SNMP qw(:snmp);
use Compress::Zlib;
use AppConfig qw(:expand :argcount);
use LWP::UserAgent;
use HTTP::Request::Common;
use XML::Simple;
use Data::Dumper;
use FindBin qw($Bin);
use File::stat;
use Parallel::ForkManager;
if (! $Config{'useithreads'}) {
	print("1..0 # Skip: Perl not compiled with 'useithreads'\n");
	exit(0);
}else {
	use threads;
	use threads::shared;
   if ($argnodisplay == 0) {
      print "Threads version: $threads::VERSION\n";
   }
	if ($threads::VERSION > 1.32){
		threads->set_stack_size(20*8192);
		my $thread_version="new";
	} else {
      if ($argnodisplay == 0) {
         print "Perl is compiled with old version of thread, this script is run in degraded mod and can crash often\n";
      }
		my $thread_version="old";
	}
}

$| = 1;

#$ENV{HTTPS_CA_FILE}  = 'cacert.pem';
#$ENV{HTTPS_CERT_FILE} = 'cacert.pem';

###########################################
######### Initial values of Agent #########
###########################################

my $experimental = 0;
my $agent_version = '1.5.3';
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$hour  = sprintf("%02d", $hour);
$min  = sprintf("%02d", $min);
$yday = sprintf("%04d", $yday);
my $PID = $yday.$hour.$min;
my $max_procs;
my $pm;
my $description;

###########################################
##### Include serials for discovery
###########################################

require $Bin.'/inc/errors.pm';
require $Bin.'/inc/device_serials.pm';
require $Bin.'/inc/functions.pm';
require $Bin.'/inc/communications_serveur.pm';
require $Bin.'/inc/tracker_snmp.pm';
require $Bin.'/inc/tracker_xml.pm';

# Manufacturer specifications
require $Bin.'/inc/devices/cisco.pm';
require $Bin.'/inc/devices/hp.pm';
require $Bin.'/inc/devices/samsung.pm';
require $Bin.'/inc/devices/wyse.pm';

require $Bin.'/inc/tracker_discovery.pm';
require $Bin.'/inc/tracker_query.pm';

require $Bin.'/inc/tracker_pid.pm';

pid_check_lock();

############################################
############# Read config file #############
############################################

if (! -e $Bin.'/'.$conffile) {
	ErrorCode('1051');
   delete_pid();
	exit;
}
my $config = AppConfig->new();
$config->define("server", {ARGCOUNT => ARGCOUNT_ONE});
$config->define("id", {ARGCOUNT => ARGCOUNT_ONE});
$config->define("key", {ARGCOUNT => ARGCOUNT_ONE});
$config->file($Bin.'/'.$conffile);

#############################################
####### Get values by http request ##########
#############################################

my $xml_file = get_snmp_http($PID,$config);

############################################
########## transform in variables ##########
############################################

my $xml = new XML::Simple;
my $data = $xml->XMLin($xml_file);
undef $xml_file;
#print Dumper($data);

my $num_files = 0;

my $xml_post = "<snmp>\n";

	#########################################
	###### Get agent config from GLPI #######
	#########################################
	
#	$glpi_agent_id = $data->{agent}->{id};
	my $nb_core_discovery = $data->{agent}->{core_discovery};
	my $nb_threads_discovery = $data->{agent}->{threads_discovery};
	my $nb_core_query = $data->{agent}->{core_query};
	my $nb_threads_query = $data->{agent}->{threads_query};
#	$glpi_agent_logs = $data->{agent}->{logs};
#	$glpi_agent_key = $data->{agent}->{key};
	$PID = $data->{agent}->{PID};
	my $fragment = ($data->{agent}->{fragment}) * 1000;
	my $log = $data->{agent}->{logs};

	debug($log,"=========================================","",$PID,$Bin);
	debug($log,"============== Start Agent ==============","",$PID,$Bin);
	debug($log,"=========================================","",$PID,$Bin);
	debug($log,"Operating system : ".$Config{osname},"",$PID,$Bin);
	debug($log,"Operating system version : ".$Config{osvers},"",$PID,$Bin);
	debug($log,"Operating system arch : ".$Config{archname},"",$PID,$Bin);
	debug($log,"Perl version : ".$Config{PERL_REVISION}.".".$Config{PERL_VERSION}.".".$Config{PERL_SUBVERSION},"",$PID,$Bin);
	debug($log,"Thread version : ".$threads::VERSION,"",$PID,$Bin);
	debug($log,"ForkManager version : ".$Parallel::ForkManager::VERSION,"",$PID,$Bin);
	debug($log,"SNMP version : ".$Net::SNMP::VERSION,"",$PID,$Bin);
	debug($log,"Zlib version : ".$Compress::Zlib::VERSION,"",$PID,$Bin);
	debug($log,"AppConfig version : ".$AppConfig::VERSION,"",$PID,$Bin);
	debug($log,"UserAgent version : ".$LWP::UserAgent::VERSION,"",$PID,$Bin);
	debug($log,"HTTP Request Common version : ".$HTTP::Request::Common::VERSION,"",$PID,$Bin);
	debug($log,"XML Simple version : ".$XML::Simple::VERSION,"",$PID,$Bin);
	debug($log,"Data Dumper version : ".$Data::Dumper::VERSION,"",$PID,$Bin);
	debug($log,"FindBin version : ".$FindBin::VERSION,"",$PID,$Bin);
	debug($log,"","",$PID,$Bin);
	debug($log,$data,"dump",$PID,$Bin);

	#########################################
	########### Discover devices ############
	#########################################

	my $date_start_discovery;
	my $date_end_discovery;
	my $authSNMP_discovery;

   if ($argnodisplay == 0) {
   	print "== Discovery devices ==\n";
   }

	my ($nbip,$iplist) = rangeip($data,$nb_core_discovery);

	debug($log,"==== List of IP to discover ====","",$PID,$Bin);
	debug($log,$iplist,"dump",$PID,$Bin);

	$authSNMP_discovery = discovery_auth($data);

	$date_start_discovery = date_now();
	if ($nbip != 0) {
		$xml_post .= discovery_threaded($nb_threads_discovery,$nb_core_discovery,$log,$Bin,$config->server,$PID,$fragment,$iplist,$authSNMP_discovery,$nbip,$argnodisplay,$argnoprogressbar);
	}

   $date_end_discovery = date_now();

	my $total_discovery = $nbip;

	$xml_post .= constructxmlAgent($agent_version,$config->id,"", $PID,$date_start_discovery,$date_end_discovery,$total_discovery,$num_files,"","");

   $xml_post .= "</snmp>\n";
	my $data_compressed = Compress::Zlib::memGzip($xml_post);
	send_snmp_http($data_compressed,$PID."-discovery",$config->server);


	#########################################
	########### Query each device ###########
	#########################################

	my $date_start_query;
	my $date_end_query;

   if ($argnodisplay == 0) {
   	print "== Query devices ==\n";
   }

	$date_start_query = date_now();
	$num_files = query_threaded($nb_threads_query, $data,$fragment,$PID,$nb_core_query,$log,$Bin,$config->server,$config->key,$agent_version,$argnodisplay,$argnoprogressbar);
	$date_end_query = date_now();

	#########################################
	############# Agent infos ###############
	#########################################

	$xml_post = "<snmp>\n";
	$xml_post .= constructxmlAgent($agent_version,$config->id,"", $PID,$date_start_discovery,$date_end_discovery,$total_discovery,$num_files,$date_start_query,$date_end_query);


   $xml_post .= "</snmp>\n";
   #print $xml_post;

   $data_compressed = Compress::Zlib::memGzip($xml_post);
   send_snmp_http($data_compressed,$PID,$config->server);

   debug($log,"=========================================","",$PID,$Bin);
   debug($log,"================== END ==================","",$PID,$Bin);
   debug($log,"=========================================","",$PID,$Bin);

   delete_pid();

   exit 0;
